using System;
using System.IO;
using System.Web;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;

namespace SendPmt
{
	/// <summary>
	/// Summary description for SendPmt.
	/// </summary>
	public class clsSendPmt
	{
		const string LIVE_TRANSACTION_URL = "https://www.paymentsgateway.net/cgi-bin/postAuth.pl";
		const string TEST_TRANSACTION_URL = "https://www.paymentsgateway.net/cgi-bin/posttest.pl";
		const char POSTSTRING_DELIMINATOR = '\n';
		const char EQUAL_SIGN = '=';

		public clsSendPmt()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		// make sure post string is http encoded
		private string HttpEncodingPostStrings(string strPost)
		{
			string[] postStrings = strPost.Split(POSTSTRING_DELIMINATOR);
			for(int counter=0; counter<postStrings.Length; counter++)
			{
				string[] postStringPair = postStrings[counter].Split(EQUAL_SIGN);
				if(postStringPair.Length == 2 && postStringPair[0].Trim().Length > 0)
				{
					postStrings[counter] = postStringPair[0].Trim() + "=" + HttpUtility.UrlEncode(postStringPair[1].Trim());
				}
				else if(postStringPair.Length == 1 && postStringPair[0].Trim().Length > 0)
				{
					postStrings[counter] = postStringPair[0];
				}
			}
			// add one more '&' to the "endofdata" as required by transactionprocessor
			strPost = String.Join("&", postStrings).Trim() + "&";

			return strPost;
		}
		
		public string SendPayment(string strPost, string testRun)
		{
			string result;			
			Stream stream = null;
			HttpWebRequest request;
            ServicePointManager.SecurityProtocol = (SecurityProtocolType)3072;

			strPost = HttpEncodingPostStrings(strPost);

			if(testRun.Trim().ToLower() == "live")
			{
				request = (HttpWebRequest)WebRequest.Create(LIVE_TRANSACTION_URL);
			}
			else if(testRun.Trim().ToLower() == "test")
			{
				request = (HttpWebRequest)WebRequest.Create(TEST_TRANSACTION_URL);
			}
			else
			{
				return "test or live?";
			}

			if(request == null) throw new SendPmtException("Can not create request Object!");

			byte[] bytes = Encoding.ASCII.GetBytes(strPost);

			request.Method = "POST";
			request.ContentType = "application/x-www-form-urlencoded";
			//request.ContentLength = bytes.Length;
			request.AllowWriteStreamBuffering = true;

			try
			{
				stream = request.GetRequestStream(); 
				stream.Write(bytes, 0, bytes.Length);
			}
			catch(Exception exp)
			{
				throw new SendPmtException("stream error!");
			}
			finally 
			{
				stream.Close();
				//stream = null;
			}

			HttpWebResponse response = (HttpWebResponse)request.GetResponse();

			if(response == null) throw new SendPmtException("Can not get response from resource!");

			using (StreamReader sr = 
					   new StreamReader(response.GetResponseStream()) )
			{
				result = sr.ReadToEnd();

				// Close and clean up the StreamReader				
				sr.Close();
			}

			return result;
		}
	}	

	public class SendPmtException : ApplicationException
	{
		public SendPmtException(String message) : base(message)
		{
		}
	}
}
